import time
from bs4 import BeautifulSoup
from cloudscraper import (
    create_scraper,
    Session
)
import re
from .utils import info_videotiktok
from requests.models import InvalidURL


class ssstik(Session):
    '''
    :param delay:
    ```python
    >>> tik=ssstik()
    >>> tik.get_media('....')
    [<[type:video]>, <[type:video]>, <[type:music]>]
    ```
    '''
    BASE = "https://ssstik.io"
    HEADERS = {
        "content-type": "application/x-www-form-urlencoded; charset=UTF-8",
        "hx-active-element": "submit",
        "hx-current-url": "https://ssstik.io/",
        "hx-request": "true",
        "hx-target": "target",
        "origin": "https://ssstik.io",
        "sec-fetch-dest": "",
        "sec-fetch-mode": "cors",
        "sec-fetch-site": "same-origin",
        "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 \
            (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36"}

    def __init__(self, delay: int = 10, **kwargs) -> None:
        super().__init__(**kwargs)
        self.headers = self.HEADERS
        self.cf = create_scraper(delay=delay)
        self.html = self.cf.get(self.BASE)
        while True:
            if self.html.status_code == 403:
                print('retrying request')
                self.cf = create_scraper(delay=delay)
                self.html = self.cf.get(self.BASE)
                time.sleep(5)
            else:
                break

    def get_media(self, url: str) -> list[info_videotiktok]:
        '''
        :param url:
        ```python
        >>> <ssstik object>.get_media('....')
        [<[type:video]>, <[type:video]>]
        ```
        '''
        try:
            post = self.cf.post(
                self.BASE+re.findall(
                    'hx-post=\"(.*?)\"',
                    self.html.text
                )[0],
                data={
                    "id": url,
                    "locale": "en",
                    "tt": 0,
                    "ts": 0
                }
            )
            respon = BeautifulSoup(post.text, "html.parser")
            hasil = [
                *[
                    info_videotiktok(
                        url=i,
                        Session=self.cf,
                        type='video'
                    )
                    for i in [
                        self.BASE+respon.find_all(
                            "a",
                            class_="pure-button pure-button-primary \
                                is-center u-bl dl-button download_link \
                                without_watermark")[0].get("href"),
                        respon.find_all(
                            "a",
                            class_="pure-button pure-button-primary \
                                is-center u-bl dl-button download_link \
                                without_watermark_direct")[0].get("href")
                        ]
                ],
                info_videotiktok(
                    respon.find_all(
                        "a",
                        class_="pure-button pure-button-primary is-center \
                            u-bl dl-button download_link music"
                    )[0].get("href"),
                    Session=self.cf,
                    type='music'
                )
            ]
            return hasil
        except IndexError:
            raise InvalidURL()


def Ssstik(url: str, delay=10):
    return ssstik(delay).get_media(url)



import requests
import json
from datetime import datetime
import re
from faker import Faker
fake = Faker()


class info_post(requests.Session):
    '''
    :param url: video url(tiktok)
    '''
    def __init__(self, url: str) -> None:
        super().__init__()
        self.headers = {
            "sec-ch-ua": '"Google Chrome";v="89", \
                "Chromium";v="89", \
                ";Not A Brand";v="99"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": "Linux",
            "sec-fetch-dest": "document",
            "sec-fetch-mode": "navigate",
            "sec-fetch-site": "none",
            "sec-fetch-user": "?1",
            "upgrade-insecure-requests": "1",
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) \
                AppleWebKit/537.36 (KHTML, like Gecko) \
                Chrome/89.0.4389.90 Safari/537.36",
            "X-Forwarded-For": fake.ipv4()
        }
        self.html = self.get(url)
        self.js = json.loads(
            re.search(
                r'\>(\{\"props\":.*?)\<\/script>',
                self.html.text
            ).group(1)
        )
        self.account = Account(
            (
                self.js['props']
                ['pageProps']['itemInfo']
                ['itemStruct']['author']
            )
        )
        self.video = self.js['props']['pageProps']['itemInfo']['itemStruct']
        self.cover = self.video['video']['cover']
        self.music = self.video['music']['title']
        self.caption = self.video['desc']
        self.create = datetime.fromtimestamp(self.video['createTime'])
        self.url = url
        (
            self.id,
            self.height,
            self.width,
            self.duration,
            self.ratio,
            self.bitrate
        ) = (
            self.video['video']['id'],
            self.video['video']['height'],
            self.video['video']['width'],
            self.video['video']['duration'],
            self.video['video']['ratio'],
            self.video['video']['bitrate']
        )
        self.tt_csrf_token = self.js['query']['$initialProps']['$csrfToken']
        self.aftercsrf = self.js['query']['$initialProps']['$encryptedWebid']
        self.tt_webid_v2 = self.js['query']['$initialProps']['$logId']
        self.headers.update(
            {
                'Cookie': f'tt_webid_v2={self.tt_webid_v2}; \
                    tt_csrf_token={self.tt_csrf_token}; \
                    {self.aftercsrf}'
            }
        )

    def __str__(self) -> str:
        return f"<(ID:{self.id})>"

    def __repr__(self) -> str:
        return self.__str__()


class Account:

    def __init__(self, js: dict) -> None:
        self.avatar = js['avatarThumb']
        self.username = js['uniqueId']
        self.nickname = js['nickname']
        self.signature = js['signature']
        self.create = datetime.fromtimestamp(js['createTime'])
        self.verified = js['verified']
        self.private = js["privateAccount"]

    def __repr__(self) -> str:
        return f"<(OWNER:{self.username} VERIFIED:{self.verified})>"

    def __str__(self) -> str:
        return self.__repr__()



from io import BytesIO
from typing import Optional, Union
from requests import Session


class info_videotiktok:

    def __init__(self,  url, Session: Session, type='video') -> None:
        self.json = url
        self.type = type
        self.Session = Session

    def get_size(self) -> int:
        return int(
            self.Session.get(
                self.json,
                stream=True
            ).headers["Content-Length"]
        )

    def download(self, out: Optional[str] = None) -> Union[None, BytesIO]:
        if isinstance(out, str):
            open(out, "wb").write(self.Session.get(self.json).content)
        else:
            return BytesIO(self.Session.get(self.json).content)

    def __str__(self) -> str:
        return f"<[type:{self.type}]>"

    def __repr__(self) -> str:
        return self.__str__()



from flask import (
    Flask,
    request,
    render_template,
    Response
)
from .mdown import Mdown
from .snaptik import Snaptik
from .ssstik import Ssstik
from .tikmate import Tikmate
from .scrapper import info_post
import json
import os


app = Flask(
    __name__,
    template_folder=os.path.abspath(__file__+'/../templates'),
    static_folder=os.path.abspath(__file__+'/../static')
)


@app.route('/')
def index():
    return render_template('index.html')


@app.route('/<path:path>')
def snapt(path):
    if path == 'info':
        for i in range(10):
            try:
                if not request.args.get('url'):
                    return json.dumps(
                        {'msg': 'url parameter required'},
                        indent=4
                    )
                resp = info_post(request.args['url'])
                js = Response(
                    json.dumps(
                        {
                            'account': {
                                'avatar': resp.account.avatar,
                                'username': resp.account.username,
                                'nickname': resp.account.nickname,
                                'signature': resp.account.signature,
                                'create': (
                                    (
                                        resp.account.create
                                        and
                                        resp.account.create.timestamp()
                                    ) or 0),
                                'verified': resp.account.verified
                            },
                            'music': resp.music,
                            'cover': resp.cover,
                            'caption': resp.caption,
                            'create': resp.create.timestamp(),
                            'url': resp.url,
                            'id': resp.id
                        },
                        indent=4
                    )
                )
                js.headers['Content-Type'] = 'application/json'
                return js
            except Exception as e:
                print(e)
                continue
        else:
            return Response(json.dumps({
                'msg': 'url tidak valid'
            }), headers={'Content-Type': 'application/json'})
    elif path not in ['snaptik', 'ssstik', 'tikmate', 'mdown']:
        return json.dumps({'msg': 'path tidak ditemukan'})
    if request.args.get('url'):
        try:
            service = ({
                'snaptik': Snaptik,
                'ssstik': Ssstik,
                'tikmate': Tikmate,
                'mdown': Mdown
            }).get(path, Snaptik)
            res = service(request.args['url'])
            return Response(
                json.dumps(
                    [
                        {
                            'type': i.type,
                            'url': i.json
                        } for i in res
                    ],
                    indent=4
                ),
                headers={
                    'Content-Type': 'application/json'
                }
            )
        except Exception as e:
            print(e)
            return Response(
                json.dumps({
                    'msg': 'url tidak valid'
                }, indent=4),
                headers={'Content-Type': 'application/json'}
            )
    else:
        return json.dumps({'msg': 'url parameter required'}, indent=4)



# Menghadeh Convert dari Js ke Py
# Decryptor For Snaptik/Tikmate Obfuscate

from typing import Union

alpha = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ+/"


def search(d: Union[str, list], q: Union[str, int]):
    try:
        return d.index(q)
    except Exception:
        return -1


def main(d, e, f):
    g = list(alpha)
    h = g[0:e]
    i = g[0:f]

    def freduce(a, b, c):
        if search(h, b) != -1:
            a += search(h, b) * (e ** c)
            return a
    j = reduces(freduce, list(d)[::-1], 0)
    k = ""
    while j > 0:
        k = i[j % f] + k
        j = int((j - (j % f)) / f)
    return int(k) or 0


def reduces(function, iterable, initializer=None):
    """
    [!] modified reduce function like js
    :https://www.geeksforgeeks.org/reduce-in-python/
    """
    it = iter(iterable)
    if initializer is None:
        value = next(it)
    else:
        value = function(initializer, next(it), 0)
    for index, element in enumerate(it, 1):
        value = function(value, element, index)
    return value


def decoder(h, u, n, t, e, r):
    r = ""
    i = 0
    while i < h.__len__():
        s = ""
        while h[i] != n[e]:
            s += h[i]
            i += 1
        for j in range(n.__len__()):
            s = s.replace(n[j], j.__str__())
        r += chr(main(s, e, 10) - t)
        i += 1
    return r



import unittest
from .scrapper import info_post
from .snaptik import snaptik
from .ssstik import ssstik
from .tikmate import tikmate
from .mdown import mdown


class TikTok(unittest.TestCase):
    base_url = 'https://www.tiktok.com/\
                    @tribunsumselcom/\
                    video/7020708969563917595'

    def test_snaptk(self):
        self.assertTrue(snaptik(self.base_url).get_media())

    def test_sstkio(self):
        self.assertTrue(ssstik().get_media(self.base_url))

    def test_info(self):
        self.assertTrue(info_post(self.base_url).js)

    def tikmat(self):
        self.assertTrue(tikmate().get_media(self.base_url))

    def mdown(self):
        self.assertTrue(mdown().get_media(self.base_url))


if __name__ == '__main__':
    unittest.main()



from requests.models import InvalidURL
from .decoder import decoder
from .utils import info_videotiktok
from ast import literal_eval
import re
import requests


class tikmate(requests.Session):
    BASE_URL = 'https://tikmate.online/'

    def __init__(self) -> None:
        super().__init__()
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) \
                AppleWebKit/537.36 (KHTML, like Gecko) \
                Chrome/86.0.4240.111 Safari/537.36'
        }

    def get_media(self, url: str) -> list[info_videotiktok]:
        media = self.post(
            self.BASE_URL+'abc.php',
            data={'url': url},
            headers={
                "origin": "https://tikmate.online",
                "referer": "https://tikmate.online/",
                "sec-ch-ua": '"Chromium";v="94",\
                    "Google Chrome";v="94", \
                    ";Not A Brand";v="99"',
                "sec-ch-ua-mobile": "?0",
                "sec-ch-ua-platform": "Linux",
                "sec-fetch-dest": "iframe",
                "sec-fetch-mode": "navigate",
                "sec-fetch-site": "same-origin",
                "sec-fetch-user": "?1",
                "upgrade-insecure-requests": "1",
                "user-agent": "Mozilla/5.0 (X11; Linux x86_64) \
                    AppleWebKit/537.36 (KHTML, like Gecko) \
                    Chrome/94.0.4606.81 Safari/537.36"
                }
        )
        if "'error_api_get'" in media.text:
            raise InvalidURL()
        tt = re.findall(r'\(\".*?,.*?,.*?,.*?,.*?.*?\)', media.text)
        decode = decoder(*literal_eval(tt[0]))
        return [
            info_videotiktok(
                self.BASE_URL+x,
                self) for x in re.findall(
                    r'(download.php\?token.*?)\"',
                    decode
                )
        ]


def Tikmate(url: str):
    return tikmate().get_media(url)



from .snaptik import snaptik
from .ssstik import ssstik
from .scrapper import info_post
from .tikmate import tikmate
from .mdown import mdown
from .Except import InvalidUrl

__all__ = [
    'snaptik',
    'ssstik',
    'info_post',
    'tikmate',
    'mdown',
    'InvalidUrl'
]



from sys import stderr
from ast import literal_eval
from .utils import info_videotiktok
from .Except import InvalidUrl
from requests import Session
from re import findall
from .decoder import decoder


class snaptik(Session):
    '''
    :param tiktok_url:
    ```python
    >>> tik=snaptik('url')
    >>> tik.get_media()
    [<[type:video]>, <[type:video]>]
    ```
    '''

    def __init__(self, tiktok_url: str) -> None:
        super().__init__()
        self.resp = self.get(
            'https://snaptik.app/abc.php',
            params={'url': tiktok_url, 'lang': 'en'},
            headers={
                'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) \
                    AppleWebKit/537.36 (KHTML, like Gecko) \
                    Chrome/86.0.4240.111 Safari/537.36'
            }
        )
        if 'error_api_web;' in self.resp.text or 'Error:' in self.resp.text:
            raise InvalidUrl()

    def get_media(self) -> list[info_videotiktok]:
        '''
        ```python
        >>> <snaptik object>.get_media()
        [<[type:video]>, <[type:video]>]
        ```
        '''
        stderr.flush()
        dec = decoder(*literal_eval(
            findall(
                r'\(\".*?,.*?,.*?,.*?,.*?.*?\)',
                self.resp.text
            )[0]
        ))

        stderr.flush()
        return [
            info_videotiktok(
                i,
                self
            )
            for i in set(
                map(
                    lambda x:x[0].strip('\\'),
                    findall(
                        r'\"(https?://(tikcdn\.net|snapsave\.info).*?)\"',
                        dec
                    )
                )
            )
        ]

    def __iter__(self):
        yield from self.get_media()


def Snaptik(url: str):
    return snaptik(url).get_media()




from setuptools import setup
from os import path
base_dir = path.abspath(path.dirname(__file__))
setup(
  name='tiktok_downloader',
  packages=['tiktok_downloader'],
  include_package_data=True,
  version='0.3.1',
  license='MIT',
  description='Tiktok Downloader&Scraper using bs4&requests',
  author='Krypton Byte',
  author_email='galaxyvplus6434@gmail.com',
  url='https://github.com/krypton-byte/tiktok_downloader',
  keywords=[
    'tiktok',
    'downloader',
    'scrapper',
    'tikdok-scraper',
    'tiktok-downloader'
  ],
  install_requires=[
          'bs4',
          'flask',
          'cloudscraper',
          'requests',
          'faker'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
  ],
)



from bs4 import BeautifulSoup
import requests
from requests.models import InvalidURL
from .utils import info_videotiktok


class mdown(requests.Session):
    BASE_URL = 'https://musicaldown.com/'

    def __init__(self) -> None:
        super().__init__()
        self.headers = {
            "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 \
                (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36"
        }

    def get_media(self, url: str):
        bs = BeautifulSoup(self.get(self.BASE_URL).text, 'html.parser')
        form = {
            i['name']: i['value'] for i in bs.find_all(
                'input',
                attrs={'type': 'hidden'}
            )
        }
        form.update({bs.find('input', attrs={'type': 'text'})['name']: url})
        res = self.post(
            f'{self.BASE_URL}download',
            data=form,
            headers={
                "origin": "https://musicaldown.com",
                "referer": "https://musicaldown.com/en/",
                "sec-ch-ua": '"Chromium";v="94", "Google Chrome";v="94", \
                    ";Not A Brand";v="99"',
                "sec-ch-ua-mobile": "?0",
                "sec-ch-ua-platform": "Linux",
                "sec-fetch-dest": "document",
                "sec-fetch-mode": "navigate",
                "sec-fetch-site": "same-origin",
                "sec-fetch-user": "?1",
                "upgrade-insecure-requests": "1",
                "user-agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 \
                    (KHTML, like Gecko) Chrome/94.0.4606.81 Safari/537.36"
            }
        )
        if 'err' in res.url:
            raise InvalidURL()
        return [
            info_videotiktok(
                i['href'],
                self
            ) for i in BeautifulSoup(
                res.text,
                'html.parser'
            ).find_all('a', attrs={'target': '_blank'})
        ]


def Mdown(url: str):
    return mdown().get_media(url)



import argparse
import json
import os
import sys
import requests
from .ssstik import Ssstik
from .snaptik import Snaptik
from .mdown import Mdown
from .tikmate import Tikmate
from .scrapper import info_post
from sys import stderr


arg = argparse.ArgumentParser(
    prog=f"python3 -m {os.path.dirname(__file__).split('/')[-1]}",
    description='Wellcome Maker'
)
arg.add_argument('--snaptik', action='store_true')
arg.add_argument('--ssstik', action='store_true')
arg.add_argument('--mdown', action='store_true')
arg.add_argument('--tikmate', action='store_true')
arg.add_argument('--info', action='store_true')
arg.add_argument('--url', type=str)
arg.add_argument('--server', action='store_true')
arg.add_argument('--host', type=str, default='127.0.0.1')
arg.add_argument('--debug', action='store_true')
arg.add_argument('--port', default=8000, type=int)
arg.add_argument('--json', action='store_true')
arg.add_argument('--save')
parse = arg.parse_args()
if parse.server:
    from .server import app
    app.run(host=parse.host, port=parse.port, debug=bool(parse.debug))
elif parse.url:
    service = (
        (
            parse.snaptik and Snaptik
        ) or (
            parse.ssstik and Ssstik
        ) or (
            parse.tikmate and Tikmate
        ) or (
            parse.mdown and Mdown
        ) or Snaptik
    )
    try:
        ok = service(parse.url)
        if parse.json or not(parse.save):
            print(
                json.dumps(
                    [
                        {
                            'type': i.type,
                            'url': i.json
                        } for i in ok
                    ],
                    indent=4
                )
            )
        elif parse.save:
            ok[0].download(parse.save)
        else:
            os.system("python3 -m tiktok_downloader --help")
    except Exception as e:
        print(e)
        stderr.write('Post Not Found\n')
        stderr.flush()
        sys.exit(1)
elif parse.info:
    try:
        resp = info_post(parse.url)
        print(
            json.dumps(
                {
                    'account': {
                        'avatar': resp.account.avatar,
                        'username': resp.account.username,
                        'nickname': resp.account.nickname,
                        'signature': resp.account.signature,
                        'create': (
                            (
                                resp.account.create
                                and
                                resp.account.create.timestamp()
                            ) or 0),
                        'verified': resp.account.verified
                    },
                    'music': resp.music,
                    'cover': resp.cover,
                    'caption': resp.caption,
                    'create': resp.create.timestamp(),
                    'url': resp.url,
                    'id': resp.id
                },
                indent=4
            )
        )
    except requests.exceptions.ConnectionError:
        stderr.write('[x] offline\n')
        stderr.flush()
        sys.exit(1)
    except (KeyError, AttributeError):
        stderr.write('Post Not Found\n')
        stderr.flush()
        sys.exit(1)
else:
    os.system("python3 -m tiktok_downloader --help")
